/*
 * Copyright (c) 2006, Your Corporation. All Rights Reserved.
 */

package gnu.io;

import bookExamples.ch15Streams.Unzipper;
import classUtils.pathUtils.NativeLibDetect;
import futils.Futil;
import gui.In;
import net.web.UrlUtils;
import utils.OsUtils;
import utils.SystemUtils;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * User: lyon
 * Date: Jan 14, 2006
 * Time: 6:09:10 AM
 * Copyright DocJava, Inc. 2005.
 */
public final class SafeCommDriver {
    private static RXTXCommDriver driver = null;

    private SafeCommDriver() {
    }

    public synchronized static CommDriver getInstance() {
        if (driver == null && NativeLibDetect.isLibInPath("rxtxSerial")) {
            driver = new RXTXCommDriver();
            driver.initialize();
        }
        return driver;
    }

    public static void main(String[] args) {
        test();
    }

    private static void test() {
        CommDriver cd = getInstance();
        if (cd == null) fixDriver();
        cd = getInstance();
        if (cd != null) System.out.println("it worked!");
        In.message("rxtxVersion="+RXTXVersion.getVersion());
        In.message("rxtxNativeVersion="+RXTXVersion.nativeGetVersion());
    }

    public static void checkDriver() {
        System.out.println("SafeCommDriver");
        CommDriver cd = getInstance();
        if (cd == null)
            badCommDriver();
        else System.out.println("driver OK!");
    }

    private static void badCommDriver() {
        if (!In.getBoolean("Could not load RXTX driver. \n" +
                "Do you want me to install it?")) return;
        if (!OsUtils.isLinux()){
            fixDriver();
        } else try {
            NativeLibs.writeLibrary();
        } catch (IOException e) {
            e.printStackTrace();

        }
    }

    /**
     * This method fixes the driver by beaming over
     * to the tmp directory a native.jar file.
     * The native.jar file is signed and its
     * native libs are beamed over to the user home directory
     * ~/.rxtx/ <br>
     * Finally, the java.library.path is altered to include <br>
     * ~/.rxtx/
     */
    private static void fixDriver() {
        File tmpDir = new File(
                SystemUtils.getTmpDir() + SystemUtils.getDirectorySeparator() + "native.jar");
        try {
            UrlUtils.getUrl(getResourceUrl(), tmpDir);
            String nativeLib = getNativeLibName();
            Unzipper uz = new Unzipper(tmpDir);
            uz.verify();
            byte b[] = uz.getBlob(nativeLib);
            if (b == null) {
                System.out.println("could not get:" + nativeLib);
                return;
            }
            File f = getRxtxHome();
            Futil.writeBytes(getRxtxLibFile( ), b);
            SystemUtils.appendJavaLibraryPath(f.getAbsolutePath());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static File getRxtxLibFile() {
        return new File(getRxtxHome(), getNativeLibName());
    }

    private static String getNativeLibName() {
        return System.mapLibraryName("rxtxSerial");
    }

    private static File getRxtxHome() {
        File f = new File(SystemUtils.getUserHome() +
                SystemUtils.getDirectorySeparator() + ".rxtx");
        if (f.exists() && f.canWrite()) return f;
        if (f.mkdir()) return f;
        System.out.println("ER:Could not createRxtxHome!");
        return f;
    }

    private static URL getResourceUrl() throws MalformedURLException {
        String rxtxUrl = "http://show.docjava.com:8086/" +
                "book/cgij/code/jnlp/libs/rxtx/";
        if (OsUtils.isLinux())
            return new URL(rxtxUrl + "linux/native.jar");
        if (OsUtils.isMacOs())
            return new URL(rxtxUrl + "mac/native.jar");
        if (OsUtils.isWindows())
            return new URL(rxtxUrl + "windows/native.jar");
        System.out.println("no automatic install supported for this platform. " +
                "Please e-mail lyon@docjava.com with a bug report");
        return null;
    }

}
