/*
 * Copyright (c) 2006, Your Corporation. All Rights Reserved.
 */

package org.jdesktop.jdic.screensaver;

import org.jdesktop.jdic.screensaver.bouncingline.BouncingLine;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Encapsulates a screensaver in a frame.  This is a convenient way to
 * debug a screensaver without having to build a native version.  It also
 * provides a way to preview a screensaver on a platform that does not
 * currently have a native adapter provided.  To use, run ScreensaverFrame's
 * main() method with the name of the screensaver class and the parameters
 * to pass to the screensaver.
 *
 * @author Mark Roth
 */
public class ScreensaverFrame
        extends JFrame {

    /**
     * Creates a frame to show the given screensaver.  Starts the screensaver
     * right away.
     *
     * @param screensaver The screensaver to show in this frame

     */
    public ScreensaverFrame(ScreensaverBase screensaver) {
        initComponents();

        setSize(640, 480);
        setLocationRelativeTo(null);

        // Initialize and set context
        this.screensaver = screensaver;
        ScreensaverContext context = createContext();
        screensaver.baseInit(context);

        // Create timer to call render loop
        // XXX - Delay hard-coded to 60 FPS for now
        this.timer = new Timer(1000 / 60,
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        ScreensaverFrame.this.screensaver.renderFrame();
                    }
                });
        this.timer.start();
    }

    /**
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        setTitle("SaverBeans Screensaver Frame");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm();
            }
        });

        pack();
    }
    // </editor-fold>//GEN-END:initComponents

    /**
     * Exit the Application
     */
    private void exitForm() {//GEN-FIRST:event_exitForm
        timer.stop();
        screensaver.baseDestroy();
        dispose();
    }//GEN-LAST:event_exitForm

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        try {

            String screensaverClassName = BouncingLine.class.getName();
            Class screensaverClass = Class.forName(screensaverClassName);
            startScreensaver(screensaverClass, screensaverClassName);
        }
        catch (ClassNotFoundException e) {
            System.err.println("Can't find screensaver " + e.getMessage());
        }
        catch (InstantiationException e) {
            System.err.println("Can't instantiate screensaver: " +
                    e.getMessage());
        }
        catch (IllegalAccessException e) {
            System.err.println("Can't instantiate screensaver: " +
                    e.getMessage());
        }
    }

    public static void startScreensaver(Class screensaverClass, String screensaverClassName)
            throws InstantiationException, IllegalAccessException {
        if (!ScreensaverBase.class.isAssignableFrom(screensaverClass)) {
            System.err.println("Error: Class " + screensaverClassName +
                    " is not a subclass of ScreensaverBase");
            System.exit(1);
        }
        ScreensaverBase screensaver =
                (ScreensaverBase) screensaverClass.newInstance();


        new ScreensaverFrame(screensaver).setVisible(true);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

    /**
     * The screensaver being run in this frame
     */
    private ScreensaverBase screensaver;

    /**
     * Delay timer for animation
     */
    private Timer timer;

    /**
     * Creates the screensaver context object
     */
    private ScreensaverContext createContext() {
        ScreensaverContext result = new ScreensaverContext();
        result.setComponent(getContentPane());
        return result;
    }

}
